# Generated by Django 2.2.5 on 2024-06-15 02:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('blogs', '0004_auto_20240615_1044'),
    ]

    operations = [
        migrations.AddField(
            model_name='blog',
            name='release_date',
            field=models.DateField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='blog',
            name='thumbnail',
            field=models.ImageField(blank=True, null=True, upload_to='thumbnail', verbose_name='サムネイル（空欄可）'),
        ),
    ]



import markdown
from django.db import models  # type: ignore
from django.utils import timezone  # type: ignore
from django.utils.text import slugify  # type: ignore
from markdownx.models import MarkdownxField  # type: ignore
from markdownx.settings import MARKDOWNX_MARKDOWN_EXTENSIONS as EXTENSIONS  # type: ignore


class Category(models.Model):
    """カテゴリーモデル"""

    # カラーリスト
    COLORS = [
        ["gray", "gray"],
        ["crimson", "crimson"],
        ["royalblue", "royalblue"],
        ["darkgreen", "darkgreen"],
        ["darkolivegreen", "darkolivegreen"],
        ["darkorange", "darkorange"],
        ["darkviolet", "darkviolet"],
    ]
    title = models.CharField("カテゴリー", max_length=20)
    thumbnail = models.ImageField(
        "サムネイル（空欄可）",
        upload_to="blogs/category/thumbnail",
        null=True,
        blank=True,
    )
    created_at = models.DateField(auto_now_add=True)
    updated_at = models.DateField(auto_now=True)
    overview = models.CharField("概要", null=True, blank=True, max_length=256)
    slug = models.SlugField(null=True, blank=True, unique=True)
    color = models.CharField(default=COLORS[0][0], choices=COLORS, max_length=50)

    def __str__(self):
        return self.title

    def save(self, *args, **kwargs):
        # slug変数が空だった場合、title変数のデータをslug変数に代入する
        if not self.slug:
            self.slug = slugify(self.title, allow_unicode=True)
        super().save(*args, **kwargs)

    class Meta:
        verbose_name = "ブログカテゴリーリスト"
        verbose_name_plural = "ブログカテゴリーリスト"


class Blog(models.Model):
    """記事作成モデル"""

    title = models.CharField("タイトル", max_length=150)
    text = MarkdownxField("テキスト", help_text="Markdown形式で書いてください。")
    category = models.ForeignKey(Category, on_delete=models.PROTECT)
    created_at = models.DateField(auto_now_add=True)
    updated_at = models.DateField(auto_now=True)
    is_publick = models.BooleanField(
        "選択",
        choices=(
            (True, "公開"),
            (False, "非公開"),
        ),
    )
    release_date = models.DateField(null=True, blank=True)
    thumbnail = models.ImageField(
        "サムネイル（空欄可）",
        upload_to="blogs/blog/thumbnail",
        null=True,
        blank=True,
    )

    def to_release(self):
        self.is_publick = True
        self.save()

    def to_private(self):
        self.is_publick = False
        self.save()

    def get_toc(self):
        md = markdown.Markdown(extensions=EXTENSIONS)
        html = md.convert(self.text)
        return md.toc  # type: ignore

    def __str__(self):
        return self.title

    def save(self, *args, **kwargs):
        if self.is_publick and not self.release_date:
            self.release_date = timezone.now()
        super().save(*args, **kwargs)

    class Meta:
        verbose_name = "ブログリスト"
        verbose_name_plural = "ブログリスト"


# class Popular(models.Model):
#     """
#     GoogleAnalytics APIモデル
#     blogsアプリで使用しているので、後にblogsアプリに移行
#
#     """
#     title = models.CharField('人気記事', max_length=100)
#     path = models.CharField('URL', max_length=100)
#     view = models.IntegerField('閲覧数')
#
#     def __str__(self):
#         return self.title
#
#     class Meta:
#         verbose_name = '人気記事リスト'
#         verbose_name_plural = '人気記事リスト'



# Generated by Django 2.2.5 on 2024-07-03 00:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('blogs', '0006_auto_20240618_1317'),
    ]

    operations = [
        migrations.AddField(
            model_name='category',
            name='color',
            field=models.CharField(choices=[['gray', 'gray'], ['crimson', 'crimson'], ['royalblue', 'royalblue'], ['darkgreen', 'darkgreen'], ['darkolivegreen', 'darkolivegreen'], ['darkorange', 'darkorange'], ['darkviolet', 'darkviolet']], default='gray', max_length=50),
        ),
    ]



# Generated by Django 2.2.5 on 2023-08-30 02:01

from django.db import migrations, models
import django.db.models.deletion
import markdownx.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=20, verbose_name='カテゴリー')),
                ('thumbnail', models.ImageField(blank=True, null=True, upload_to='thumbnail', verbose_name='サムネイル（空欄可）')),
            ],
            options={
                'verbose_name': 'ブログカテゴリーリスト',
                'verbose_name_plural': 'ブログカテゴリーリスト',
            },
        ),
        migrations.CreateModel(
            name='Popular',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=100, verbose_name='人気記事')),
                ('path', models.CharField(max_length=100, verbose_name='URL')),
                ('view', models.IntegerField(verbose_name='閲覧数')),
            ],
            options={
                'verbose_name': '人気記事リスト',
                'verbose_name_plural': '人気記事リスト',
            },
        ),
        migrations.CreateModel(
            name='Blog',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=150, verbose_name='タイトル')),
                ('text', markdownx.models.MarkdownxField(help_text='Markdown形式で書いてください。', verbose_name='テキスト')),
                ('created_at', models.DateField(auto_now_add=True)),
                ('updated_at', models.DateField(auto_now=True)),
                ('is_publick', models.BooleanField(choices=[(True, '公開'), (False, '非公開')], verbose_name='選択')),
                ('category', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='blogs.Category')),
            ],
            options={
                'verbose_name': 'ブログリスト',
                'verbose_name_plural': 'ブログリスト',
            },
        ),
    ]



from django.contrib import admin  # type: ignore
from django.db import models  # type: ignore
from markdownx.widgets import AdminMarkdownxWidget  # type: ignore

from .models import Blog, Category


class CategoryAdmin(admin.ModelAdmin):
    list_display = ("id", "title")
    list_display_links = ("id", "title")


class BlogAdmin(admin.ModelAdmin):
    formfield_overrides = {
        models.TextField: {"widget": AdminMarkdownxWidget},
    }


admin.site.register(Category, CategoryAdmin)
admin.site.register(Blog, BlogAdmin)
# admin.site.register(Popular)



# Generated by Django 2.2.5 on 2024-06-14 04:03

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('blogs', '0002_delete_popular'),
    ]

    operations = [
        migrations.AddField(
            model_name='category',
            name='created_at',
            field=models.DateField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='category',
            name='overview',
            field=models.CharField(blank=True, max_length=256, null=True, verbose_name='概要'),
        ),
        migrations.AddField(
            model_name='category',
            name='slug',
            field=models.SlugField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='category',
            name='updated_at',
            field=models.DateField(auto_now=True),
        ),
    ]



# Generated by Django 2.2.5 on 2024-06-15 01:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('blogs', '0003_auto_20240614_1303'),
    ]

    operations = [
        migrations.AlterField(
            model_name='category',
            name='slug',
            field=models.SlugField(blank=True, null=True, unique=True),
        ),
    ]



from django.contrib.auth.models import User  # type: ignore
from django.test import TestCase  # type: ignore
from django.urls import reverse  # type: ignore

from .models import Category


class IndexViewTests(TestCase):
    """IndexViewのテスト"""

    def test_zero_blog(self):
        """ブログ記事存在しない場合のindex"""
        response = self.client.get(reverse("blogs:index"))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context["blogs"], [])

    def test_up_blog_private(self):
        """非公開ブログ記事存在する場合のindex"""
        category = Category.objects.create(title="カテゴリー１")
        category.blog_set.create(title="タイトル", text="テキスト", is_publick=False)
        response = self.client.get(reverse("blogs:index"))
        self.assertQuerysetEqual(response.context["blogs"], [])

    def test_up_blog_publick(self):
        """公開ブログ記事存在する場合のindexページの見え方"""
        category = Category.objects.create(title="カテゴリー１")
        category.blog_set.create(title="タイトル", text="テキスト", is_publick=True)
        response = self.client.get(reverse("blogs:index"))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context["blogs"], ["<Blog: タイトル>"])
        # QuerySetを使用した判定の場合
        # queryset = category.blog_set.filter(is_publick=True).order_by('-id')
        # self.assertQuerysetEqual(
        #         response.context["blogs"],
        #         queryset,
        #         transform=lambda x: x, # デフォルトではtransform=repr()が使用されているためエラーとなるので変更。
        # )


class CategoryViewTests(TestCase):
    """CategoryViewのテスト"""

    def test_zero_category(self):
        """ブログカテゴリーの値が存在しない場合に/blogs/category/***にアクセスした際の判定"""
        url = reverse("blogs:category", args=("カテゴリー１",))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

    def test_up_category(self):
        """ブログカテゴリーの値が存在する場合の/blogs/category/***にアクセスした際の判定"""
        category = Category.objects.create(title="カテゴリー１")
        url = reverse("blogs:category", args=("カテゴリー１",))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, category.title)
        self.assertQuerysetEqual(response.context["blogs"], [])

    def test_up_blog_in_category(self):
        """ブログ記事が存在する場合の/blogs/category/***にアクセスした際の判定"""
        category = Category.objects.create(title="カテゴリー１")
        category.blog_set.create(title="タイトル", text="テキスト", is_publick=True)
        url = reverse("blogs:category", args=("カテゴリー１",))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, category.title)
        self.assertQuerysetEqual(response.context["blogs"], ["<Blog: タイトル>"])
        # QuerySetを使用した判定の場合
        # queryset = Blog.objects.filter(is_publick=True, category=category).order_by("-id")
        # self.assertQuerysetEqual(
        #         response.context["blogs"],
        #         queryset,
        #         transform=lambda x: x, # デフォルトではtransform=repr()が使用されているためエラーとなるので変更。
        # )


class BlogDetailViewTests(TestCase):
    """BlogDetailViewのテスト"""

    def test_detail_result(self):
        """ブログ記事の詳細ページの結果"""
        category = Category.objects.create(title="カテゴリー１")
        category.blog_set.create(title="タイトル", text="テキスト", is_publick=True)
        blog = category.blog_set.get(title="タイトル")
        url = reverse("blogs:detail", args=(blog.id,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, blog.title)


class PrivateIndexViewTests(TestCase):
    """PrivateIndexViewクラスのテスト"""

    def test_no_account_result(self):
        """権限の無い状態からアクセスした場合"""
        response = self.client.get(reverse("blogs:private_index"))
        self.assertContains(response, "権限がありません。")

    def test_login_user_result(self):
        """権限があり、非公開ブログ記事が存在する場合"""
        category = Category.objects.create(title="カテゴリー１")
        category.blog_set.create(title="タイトル", text="テキスト", is_publick=False)
        queryset = category.blog_set.filter(is_publick=False).order_by("-id")
        self.client.force_login(User.objects.create_user("tester"))
        response = self.client.get(reverse("blogs:private_index"))
        self.assertQuerysetEqual(
            response.context["private_blog"], queryset, transform=lambda x: x
        )


class PrivateDetailViewTests(TestCase):
    """PrivateDetailViewのテスト"""

    def test_no_account_result(self):
        """権限の無い状態からアクセスした場合"""
        category = Category.objects.create(title="カテゴリー１")
        category.blog_set.create(title="タイトル", text="テキスト", is_publick=False)
        blog = category.blog_set.get(is_publick=False)
        url = reverse("blogs:private_detail", args=(blog.id,))
        response = self.client.get(url)
        self.assertContains(response, "権限がありません。")

    def test_login_user_private_detail_result(self):
        """非公開ブログ記事の詳細ページの結果"""
        category = Category.objects.create(title="カテゴリー１")
        category.blog_set.create(title="タイトル", text="テキスト", is_publick=False)
        blog = category.blog_set.get(is_publick=False)
        self.client.force_login(User.objects.create_user("tester"))
        url = reverse("blogs:private_detail", args=(blog.id,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, blog.title)


class BlogFormViewTests(TestCase):
    """BlogFormViewのテスト"""

    def test_blog_post_redirect(self):
        """ブログが保存され、リダイレクト先へ遷移するテスト"""
        Category.objects.create(title="カテゴリー１")
        response = self.client.post(
            path=reverse("blogs:new_blog"),
            data={"category": 1, "title": "タイトル", "text": "テキスト", "is_publick": True},
        )
        redirect_url = reverse("blogs:index")
        self.assertRedirects(response, redirect_url)

    def test_no_account_new_blog_access(self):
        """権限の無い状態からブログ作成ページにアクセスした場合"""
        response = self.client.get(reverse("blogs:new_blog"))
        self.assertContains(response, "権限がありません。")

    def test_login_user_new_blog_access(self):
        """ログインユーザーがブログ作成ページにアクセスした結果"""
        self.client.force_login(User.objects.create_user("tester"))
        response = self.client.get(reverse("blogs:new_blog"))
        self.assertContains(response, "戻る")


class EditBlogFormViewTests(TestCase):
    """EditBlogFormViewのテスト"""

    def test_blog_edit_redirect(self):
        """ブログが更新され、リダイレクト先へ遷移するテスト"""
        category = Category.objects.create(title="カテゴリー１")
        blog = category.blog_set.create(title="タイトル", text="テキスト", is_publick=False)
        url = reverse("blogs:edit", args=(blog.id,))
        redirect_url = reverse("blogs:index")
        response = self.client.post(
            path=url,
            data={"category": 1, "title": "タイトル", "text": "テキスト", "is_publick": True},
        )
        self.assertRedirects(response, redirect_url)

    def test_from_no_account_editblog_access(self):
        """権限の無い状態からブログ更新ページにアクセスした場合"""
        category = Category.objects.create(title="カテゴリー１")
        blog = category.blog_set.create(title="タイトル", text="テキスト", is_publick=False)
        url = reverse("blogs:edit", args=(blog.id,))
        response = self.client.get(url)
        self.assertContains(response, "権限がありません。")

    def test_from_login_user_editblog_access(self):
        """ログインユーザーがブログ更新ページにアクセスした結果"""
        category = Category.objects.create(title="カテゴリー１")
        blog = category.blog_set.create(title="タイトル", text="テキスト", is_publick=False)
        self.client.force_login(User.objects.create_user("tester"))
        url = reverse("blogs:edit", args=(blog.id,))
        response = self.client.get(url)
        self.assertContains(response, "戻る")


class ReleaseTests(TestCase):
    """release関数のテスト"""

    def test_blog_release_redirect(self):
        """特定のブログ記事が公開にアクセスされた際のリダイレクト先へ遷移するテスト"""
        category = Category.objects.create(title="カテゴリー１")
        blog = category.blog_set.create(title="タイトル", text="テキスト", is_publick=False)
        self.client.force_login(User.objects.create_user("tester"))
        url = reverse("blogs:release", args=(blog.id,))
        redirect_url = reverse("blogs:index")
        response = self.client.get(url)
        self.assertRedirects(response, redirect_url)

    def test_no_account_blog_release_access(self):
        """権限の無い状態から特定のブログ記事の公開にアクセスされた際のHttpResponse"""
        category = Category.objects.create(title="カテゴリー１")
        blog = category.blog_set.create(title="タイトル", text="テキスト", is_publick=False)
        url = reverse("blogs:release", args=(blog.id,))
        response = self.client.get(url)
        self.assertContains(response, "権限がありません。")


class PrivateTests(TestCase):
    """private関数のテスト"""

    def test_blog_private_redirect(self):
        """特定のブログ記事が非公開にアクセスされた際のリダイレクト先へ遷移するテスト"""
        category = Category.objects.create(title="カテゴリー１")
        blog = category.blog_set.create(title="タイトル", text="テキスト", is_publick=True)
        self.client.force_login(User.objects.create_user("tester"))
        url = reverse("blogs:private", args=(blog.id,))
        redirect_url = reverse("blogs:private_index")
        response = self.client.get(url)
        self.assertRedirects(response, redirect_url)

    def test_no_account_blog_private_access(self):
        """権限の無い状態から特定のブログ記事の非公開にアクセスされた際のHttpResponse"""
        category = Category.objects.create(title="カテゴリー１")
        blog = category.blog_set.create(title="タイトル", text="テキスト", is_publick=False)
        url = reverse("blogs:private", args=(blog.id,))
        response = self.client.get(url)
        self.assertContains(response, "権限がありません。")



from django import template  # type: ignore
from django.utils.safestring import mark_safe  # type: ignore
from markdownx.utils import markdownify  # type: ignore

register = template.Library()


@register.filter
def markdown_to_html(text):
    return mark_safe(markdownify(text))


# class EscapeHtml(Extension):
#     def extendMarkdown(self, md):
#         md.preprocessors.deregister('html_block')
#         md.inlinePatterns.deregister('html')
